Structure Vector3
	X.f
	Y.f
	Z.f
EndStructure

Structure Axis3
	Pitch.f
	Yaw.f
	Roll.f
EndStructure

Structure Matrix3
	A11.f : A12.f : A13.f
	A21.f : A22.f : A23.f
	A31.f : A32.f : A33.f 
EndStructure




Procedure M3_Multiply(*Use.Matrix3, *Source.Matrix3)
	Protected Buffer.Matrix3
	CopyMemory(*Use, @Buffer, SizeOf(Matrix3)) 
	With *Use
		\A11 = Buffer\A11 * *Source\A11 + Buffer\A12 * *Source\A21 + Buffer\A13 * *Source\A31
		\A12 = Buffer\A11 * *Source\A12 + Buffer\A12 * *Source\A22 + Buffer\A13 * *Source\A32
		\A13 = Buffer\A11 * *Source\A13 + Buffer\A12 * *Source\A23 + Buffer\A13 * *Source\A33
		\A21 = Buffer\A21 * *Source\A11 + Buffer\A22 * *Source\A21 + Buffer\A23 * *Source\A31
		\A22 = Buffer\A21 * *Source\A12 + Buffer\A22 * *Source\A22 + Buffer\A23 * *Source\A32
		\A23 = Buffer\A21 * *Source\A13 + Buffer\A22 * *Source\A23 + Buffer\A23 * *Source\A33
		\A31 = Buffer\A31 * *Source\A11 + Buffer\A32 * *Source\A21 + Buffer\A33 * *Source\A31
		\A32 = Buffer\A31 * *Source\A12 + Buffer\A32 * *Source\A22 + Buffer\A33 * *Source\A32
		\A33 = Buffer\A31 * *Source\A13 + Buffer\A32 * *Source\A23 + Buffer\A33 * *Source\A33
	EndWith
	ProcedureReturn *Use
EndProcedure

Procedure M3_Multiplication(*Use.Matrix3, *Source1.Matrix3, *Source2.Matrix3)
	Protected Buffer.Matrix3
	If *Use = *Source1
		CopyMemory(*Source1, @Buffer, SizeOf(Matrix3)) 
		*Source1 = @Buffer
	EndIf	
	If *Use = *Source2
		CopyMemory(*Source2, @Buffer, SizeOf(Matrix3))
		*Source2 = @Buffer
	EndIf
	With *Use
		\A11 = *Source1\A11 * *Source2\A11 + *Source1\A12 * *Source2\A21 + *Source1\A13 * *Source2\A31
		\A12 = *Source1\A11 * *Source2\A12 + *Source1\A12 * *Source2\A22 + *Source1\A13 * *Source2\A32
		\A13 = *Source1\A11 * *Source2\A13 + *Source1\A12 * *Source2\A23 + *Source1\A13 * *Source2\A33
		\A21 = *Source1\A21 * *Source2\A11 + *Source1\A22 * *Source2\A21 + *Source1\A23 * *Source2\A31
		\A22 = *Source1\A21 * *Source2\A12 + *Source1\A22 * *Source2\A22 + *Source1\A23 * *Source2\A32
		\A23 = *Source1\A21 * *Source2\A13 + *Source1\A22 * *Source2\A23 + *Source1\A23 * *Source2\A33
		\A31 = *Source1\A31 * *Source2\A11 + *Source1\A32 * *Source2\A21 + *Source1\A33 * *Source2\A31
		\A32 = *Source1\A31 * *Source2\A12 + *Source1\A32 * *Source2\A22 + *Source1\A33 * *Source2\A32
		\A33 = *Source1\A31 * *Source2\A13 + *Source1\A32 * *Source2\A23 + *Source1\A33 * *Source2\A33
	EndWith
	ProcedureReturn *Use
EndProcedure

Procedure M3_Rotation(*Use.Matrix3, X.f, Y.f, Z.f)
	Protected CosZ.f = Cos(Z), CosY.f = Cos(Y), CosX.f = Cos(X)
	Protected SinZ.f = Sin(Z), SinY.f = Sin(Y), SinX.f = Sin(X)
	With *Use
		\A11 =  CosY*CosZ                : \A12 = -CosY*SinZ                : \A13 =  SinY
		\A21 =  SinX*SinY*CosZ+CosX*SinZ : \A22 = -SinX*SinY*SinZ+CosX*CosZ : \A23 = -SinX*CosY
		\A31 = -CosX*SinY*CosZ+SinX*SinZ : \A32 =  CosX*SinY*SinZ+SinX*CosZ : \A33 =  CosX*CosY
; 		\A11 =  CosY*CosZ : \A12 =  SinX*SinY*CosZ-CosX*SinZ : \A13 =  CosX*SinY*CosZ+SinX*SinZ 
; 		\A21 =  CosY*SinZ : \A22 =  SinX*SinY*SinZ+CosX*CosZ : \A23 =  CosX*SinY*SinZ-SinX*CosZ
; 		\A31 = -SinY      : \A32 =  SinX*CosY                : \A33 =  CosX*CosY
	EndWith
	ProcedureReturn *Use
EndProcedure

Procedure.s M3_Get(*Source.Matrix3)
	ProcedureReturn StrF(*Source\A11)+", "+StrF(*Source\A12)+", "+StrF(*Source\A13)+" ; "+StrF(*Source\A21)+", "+StrF(*Source\A22)+", "+StrF(*Source\A23)+" ; "+StrF(*Source\A31)+", "+StrF(*Source\A32)+", "+StrF(*Source\A33)
EndProcedure




Procedure.i V3_Set(*Use.Vector3, X.f, Y.f, Z.f)
	*Use\X = X
	*Use\Y = Y
	*Use\Z = Z
	ProcedureReturn *Use
EndProcedure



Procedure.i V3_Add(*Use.Vector3, *Source.Vector3, Factor.f=1.0)
	*Use\X + *Source\X*Factor
	*Use\Y + *Source\Y*Factor
	*Use\Z + *Source\Z*Factor
	ProcedureReturn *Use
EndProcedure

Procedure.i V3_Subtract(*Use.Vector3, *Source.Vector3, Factor.f=1.0)
	*Use\X - *Source\X*Factor
	*Use\Y - *Source\Y*Factor
	*Use\Z - *Source\Z*Factor
	ProcedureReturn *Use
EndProcedure

Procedure.i V3_Multiply(*Use.Vector3, Value.f)
	*Use\X * Value
	*Use\Y * Value
	*Use\Z * Value
	ProcedureReturn *Use
EndProcedure

Procedure.i V3_Divide(*Use.Vector3, Value.f)
	*Use\X / Value
	*Use\Y / Value
	*Use\Z / Value
	ProcedureReturn *Use
EndProcedure



Procedure.i V3_Addition(*Use.Vector3, *Source1.Vector3, *Source2.Vector3)
	*Use\X = *Source1\X + *Source2\X
	*Use\Y = *Source1\Y + *Source2\Y
	*Use\Z = *Source1\Z + *Source2\Z
	ProcedureReturn *Use
EndProcedure

Procedure.i V3_Subtraction(*Use.Vector3, *Source1.Vector3, *Source2.Vector3)
	*Use\X = *Source1\X - *Source2\X
	*Use\Y = *Source1\Y - *Source2\Y
	*Use\Z = *Source1\Z - *Source2\Z
	ProcedureReturn *Use
EndProcedure

Procedure.i V3_Multiplication(*Use.Vector3, *Source.Vector3, Value.f)
	*Use\X = *Source\X * Value
	*Use\Y = *Source\Y * Value
	*Use\Z = *Source\Z * Value
	ProcedureReturn *Use
EndProcedure

Procedure.i V3_Division(*Use.Vector3, *Source.Vector3, Value.f)
	*Use\X = *Source\X / Value
	*Use\Y = *Source\Y / Value
	*Use\Z = *Source\Z / Value
	ProcedureReturn *Use
EndProcedure



Procedure.f V3_Length(*Source.Vector3)
	ProcedureReturn Sqr(*Source\X * *Source\X + *Source\Y * *Source\Y + *Source\Z * *Source\Z)
EndProcedure

Procedure.f V3_Scalar(*Source1.Vector3, *Source2.Vector3)
	ProcedureReturn *Source1\X * *Source2\X + *Source1\Y * *Source2\Y + *Source1\Z * *Source2\Z
EndProcedure

Procedure.f V3_Angle(*Source1.Vector3, *Source2.Vector3)
	ProcedureReturn ACos(V3_Scalar(*Source1, *Source2)/(V3_Length(*Source1)*V3_Length(*Source2)))
EndProcedure

Procedure.f V3_Distance(*Source1.Vector3, *Source2.Vector3)
	ProcedureReturn Sqr((*Source1\X-*Source2\X) * (*Source1\X-*Source2\X) + (*Source1\Y-*Source2\Y) * (*Source1\Y-*Source2\Y) + (*Source1\Z-*Source2\Z) * (*Source1\Z-*Source2\Z))
EndProcedure




Procedure.i V3_Cross(*Use.Vector3, *Source1.Vector3, *Source2.Vector3)
	*Use\X = *Source1\Y * *Source2\Z - *Source1\Z * *Source2\Y
	*Use\Y = *Source1\Z * *Source2\X - *Source1\X * *Source2\Z
	*Use\Z = *Source1\X * *Source2\Y - *Source1\Y * *Source2\X
	ProcedureReturn *Use
EndProcedure

Procedure.i V3_Normalize(*Use.Vector3)
	Protected Length.f = V3_Length(*Use)
	If Length
		*Use\X / Length
		*Use\Y / Length
		*Use\Z / Length
	EndIf
	ProcedureReturn *Use
EndProcedure

Procedure.i V3_Rotate(*Use.Vector3, *Source.Matrix3)
	Protected Buffer.Vector3 
	CopyMemory(*Use, @Buffer, SizeOf(Vector3)) 
	*Use\X = Buffer\X**Source\A11 + Buffer\Y**Source\A12 + Buffer\Z**Source\A13
	*Use\Y = Buffer\X**Source\A21 + Buffer\Y**Source\A22 + Buffer\Z**Source\A23
	*Use\Z = Buffer\X**Source\A31 + Buffer\Y**Source\A32 + Buffer\Z**Source\A33
	ProcedureReturn *Use
EndProcedure

Procedure.i V3_Rotation(*Use.Vector3, *Source1.Matrix3, *Source2.Vector3 )
	*Use\X = *Source2\X**Source1\A11 + *Source2\Y**Source1\A12 + *Source2\Z**Source1\A13
	*Use\Y = *Source2\X**Source1\A21 + *Source2\Y**Source1\A22 + *Source2\Z**Source1\A23
	*Use\Z = *Source2\X**Source1\A31 + *Source2\Y**Source1\A32 + *Source2\Z**Source1\A33
	ProcedureReturn *Use
EndProcedure

Procedure.i V3_SetLength(*Use.Vector3, Length.f)
	V3_Normalize(*Use)
	V3_Multiply(*Use, Length)
	ProcedureReturn *Use
EndProcedure

Procedure.s V3_Get(*Source.Vector3)
	ProcedureReturn StrF(*Source\X)+", "+StrF(*Source\Y)+", "+StrF(*Source\Z)
EndProcedure


Procedure V3_Project(*Use.Vector3, Distance.f)
	If *Use\Z
		*Use\X = *Use\X * Distance / *Use\Z
		*Use\Y = *Use\Y * Distance / *Use\Z
	EndIf
	ProcedureReturn *Use
EndProcedure
; IDE Options = PureBasic 5.00 Beta 2 (Windows - x64)
; CursorPosition = 220
; FirstLine = 159
; Folding = -----
; EnableXP
; EnableCompileCount = 0
; EnableBuildCount = 0