
XIncludeFile "Math3D.pbi"
XIncludeFile "DirectX9.pbi"




Structure Voxel
	X.f
	Y.f
	Z.f
	Color.q
	Distance.f
	*NextVoxel.Voxel
EndStructure

Structure VoxelInclude_Camera
	Position.Vector3
	Orientation.Matrix3
	Distance.f
EndStructure

Structure VoxelInclude
	Camera.VoxelInclude_Camera
	Sprite3D.i
	*DirectX9.DirectX9_Sprite3D
	Light.Vector3
	List Voxel.Voxel()
	Array *VoxelHash.Voxel(64,64,64)
	Faces.i
EndStructure

Global VoxelInclude.VoxelInclude


Procedure InitVoxel(FieldOfView.f=75)
	
	Protected Sprite.i, Sprite3D.i
	
	With VoxelInclude
		
		Sprite = CreateSprite(#PB_Any, 1, 1, #PB_Sprite_Texture)
		StartDrawing(SpriteOutput(Sprite))
			Plot(0, 0, $FFFFFF)
		StopDrawing()
		\Sprite3D = CreateSprite3D(#PB_Any, Sprite)
		\DirectX9 = IsSprite3D(\Sprite3D)
		
		V3_Set(\Camera\Position, 0, 0, #Size*5)
		M3_Rotation(\Camera\Orientation, 0, 0, 0)
		\Camera\Distance = 1/Tan(Radian(FieldOfView)/2)
		
		V3_Set(\Light, -2, 1, 1)
		V3_Normalize(\Light)
		
	EndWith
	
EndProcedure

Procedure GetVoxel(X.i, Y.i, Z.i)
	ProcedureReturn VoxelInclude\VoxelHash(X+32,Y+32,Z+32)
EndProcedure


Procedure SetVoxel(X.i, Y.i, Z.i, Color.i)
	
	Protected *Voxel.Voxel = GetVoxel(X, Y, Z)
	
	If *Voxel = #Null
		*Voxel = AddElement(VoxelInclude\Voxel())
		VoxelInclude\VoxelHash(X+32,Y+32,Z+32) = *Voxel
	EndIf
	
	With *Voxel
		\X = X
		\Y = Y
		\Z = Z
		\Color = Color
	EndWith
	
EndProcedure


Procedure Voxel_Face(*Voxel.Voxel, Array Vertex.Vector3(1), N1, N2, N3, N4, *Normal.Vector3)
	
	Protected Light.f, Cam.Vector3
	
	V3_Rotate(*Normal, VoxelInclude\Camera\Orientation)
	Light = V3_Scalar(*Normal, VoxelInclude\Light)*0.3+0.7
	VoxelInclude\DirectX9\Vertice[0]\X = Vertex(N1)\X*512 + 512
	VoxelInclude\DirectX9\Vertice[0]\Y = Vertex(N1)\Y*512 + 386
	VoxelInclude\DirectX9\Vertice[1]\X = Vertex(N2)\X*512 + 512
	VoxelInclude\DirectX9\Vertice[1]\Y = Vertex(N2)\Y*512 + 386
	VoxelInclude\DirectX9\Vertice[2]\X = Vertex(N3)\X*512 + 512
	VoxelInclude\DirectX9\Vertice[2]\Y = Vertex(N3)\Y*512 + 386
	VoxelInclude\DirectX9\Vertice[3]\X = Vertex(N4)\X*512 + 512
	VoxelInclude\DirectX9\Vertice[3]\Y = Vertex(N4)\Y*512 + 386
	For N = 0 To 3
		VoxelInclude\DirectX9\Vertice[N]\Color = RGB(Blue(*Voxel\Color)*Light, Green(*Voxel\Color)*Light, Red(*Voxel\Color)*Light)
	Next
	VoxelInclude\DirectX9\Transformed = #True
	
	If 1; V3_Scalar(*Normal, V3_Set(Cam, 0, 0, 1)) > -0.5
	DisplaySprite3D(VoxelInclude\Sprite3D, 0, 0, *Voxel\Color>>24)
	VoxelInclude\Faces + 1
	EndIf
	
EndProcedure



Procedure RenderVoxel()
	
	Protected Dim Vertex.Vector3(7), Normal.Vector3, N.i
	
	ForEach VoxelInclude\Voxel()
		With VoxelInclude\Voxel()
			\Distance = V3_Distance(V3_Rotation(Normal, VoxelInclude\Camera\Orientation, VoxelInclude\Voxel()), VoxelInclude\Camera\Position)
		EndWith
	Next
	SortStructuredList(VoxelInclude\Voxel(), #PB_Sort_Descending, OffsetOf(Voxel\Distance), #PB_Sort_Float)
	
	VoxelInclude\Faces = 0
	
	ForEach VoxelInclude\Voxel()
		With VoxelInclude\Voxel()
			
			V3_Set(Vertex(%000), \X-0.5, \Y-0.5, \Z-0.5)
			V3_Set(Vertex(%001), \X+0.5, \Y-0.5, \Z-0.5)
			V3_Set(Vertex(%010), \X-0.5, \Y+0.5, \Z-0.5)
			V3_Set(Vertex(%011), \X+0.5, \Y+0.5, \Z-0.5)
			V3_Set(Vertex(%100), \X-0.5, \Y-0.5, \Z+0.5)
			V3_Set(Vertex(%101), \X+0.5, \Y-0.5, \Z+0.5)
			V3_Set(Vertex(%110), \X-0.5, \Y+0.5, \Z+0.5)
			V3_Set(Vertex(%111), \X+0.5, \Y+0.5, \Z+0.5)
			For N = %000 To %111
				V3_Rotate(Vertex(N), VoxelInclude\Camera\Orientation)
				V3_Subtraction(Vertex(N), VoxelInclude\Camera\Position, Vertex(N))
				V3_Project(Vertex(N), VoxelInclude\Camera\Distance)
			Next
			
			If GetVoxel(\X, \Y, \Z+1) = #Null
				Voxel_Face(VoxelInclude\Voxel(), Vertex(), %111, %110, %101, %100, V3_Set(Normal, 0, 0, 1))
			EndIf
			If GetVoxel(\X, \Y, \Z-1) = #Null
				Voxel_Face(VoxelInclude\Voxel(), Vertex(), %010, %011, %000, %001, V3_Set(Normal, 0, 0, -1))
			EndIf
			If GetVoxel(\X, \Y+1, \Z) = #Null
				Voxel_Face(VoxelInclude\Voxel(), Vertex(), %111, %011, %110, %010, V3_Set(Normal, 0, 1, 0))
			EndIf
			If GetVoxel(\X, \Y-1, \Z) = #Null
				Voxel_Face(VoxelInclude\Voxel(), Vertex(), %001, %101, %000, %100, V3_Set(Normal, 0, -1, 0))
			EndIf
			If GetVoxel(\X+1, \Y, \Z) = #Null
				Voxel_Face(VoxelInclude\Voxel(), Vertex(), %111, %101, %011, %001, V3_Set(Normal, 1, 0, 0))
			EndIf
			If GetVoxel(\X-1, \Y, \Z) = #Null
				Voxel_Face(VoxelInclude\Voxel(), Vertex(), %100, %110, %000, %010, V3_Set(Normal, -1, 0, 0))
			EndIf
			
		EndWith
	Next
	
EndProcedure
; IDE Options = PureBasic 5.00 Beta 2 (Windows - x64)
; CursorPosition = 104
; FirstLine = 101
; Folding = ---
; EnableXP
; EnableCompileCount = 0
; EnableBuildCount = 0